@@ -305,6 +305,33 @@ class TestIntegerBase(unittest.TestCase):
         self.assertEqual(v3.sqrt(), 7)
         self.assertEqual(v4.sqrt(), 10**50)
 
+    def test_sqrt_module(self):
+
+        # Invalid modulus (non positive)
+        self.assertRaises(ValueError, self.Integer(5).sqrt, 0)
+        self.assertRaises(ValueError, self.Integer(5).sqrt, -1)
+
+        # Simple cases
+        assert self.Integer(0).sqrt(5) == 0
+        assert self.Integer(1).sqrt(5) in (1, 4)
+
+        # Test with all quadratic residues in several fields
+        for p in (11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53):
+            for i in range(0, p):
+                square = i**2 % p
+                res = self.Integer(square).sqrt(p)
+                assert res in (i, p - i)
+
+        # 2 is a non-quadratic reside in Z_11
+        self.assertRaises(ValueError, self.Integer(2).sqrt, 11)
+        
+        # 10 is not a prime
+        self.assertRaises(ValueError, self.Integer(4).sqrt, 10)
+
+        # 5 is square residue of 4 and 7
+        assert self.Integer(5 - 11).sqrt(11) in (4, 7)
+        assert self.Integer(5 + 11).sqrt(11) in (4, 7)
+
     def test_in_place_add(self):
         v1, v2 = self.Integers(10, 20)
 
@@ -379,9 +406,11 @@ class TestIntegerBase(unittest.TestCase):
         self.assertTrue(isinstance(v1 >> v2, self.Integer))
         self.assertEqual(v1 >> v2, 0x08)
         self.assertEqual(v1 >> 1, 0x08)
-        self.assertEqual(v3 >> 1, -0x08)
         self.assertRaises(ValueError, lambda: v1 >> -1)
-        self.assertRaises(ValueError, lambda: v1 >> (2 ** 1000))
+        self.assertEqual(v1 >> (2 ** 1000), 0)
+        
+        self.assertEqual(v3 >> 1, -0x08)
+        self.assertEqual(v3 >> (2 ** 1000), -1)
 
     def test_in_place_right_shift(self):
         v1, v2, v3 = self.Integers(0x10, 1, -0x10)
@@ -438,10 +467,15 @@ class TestIntegerBase(unittest.TestCase):
         self.assertEqual(v1.get_bit(1), 1)
         self.assertEqual(v1.get_bit(v3), 1)
         self.assertEqual(v1.get_bit(8), 1)
-        self.assertEqual(v2.get_bit(0), 1)
-        self.assertEqual(v2.get_bit(8), 1)
+        self.assertEqual(v1.get_bit(9), 0)
+
+        self.assertRaises(ValueError, v1.get_bit, -1)
+        self.assertEqual(v1.get_bit(2 ** 1000), 0)
+
         self.assertRaises(ValueError, v2.get_bit, -1)
-        self.assertRaises(ValueError, v2.get_bit, 2 ** 1000)
+        self.assertRaises(ValueError, v2.get_bit, 0)
+        self.assertRaises(ValueError, v2.get_bit, 1)
+        self.assertRaises(ValueError, v2.get_bit, 2 * 1000)
 
     def test_odd_even(self):
         v1, v2, v3, v4, v5 = self.Integers(0, 4, 17, -4, -17)
@@ -673,6 +707,21 @@ def get_tests(config={}):
     except (ImportError, OSError) as e:
         import sys
         sys.stdout.write("Skipping GMP tests (%s)\n" % str(e) )
+
+    try:
+        from Cryptodome.Math import _Numbers_custom as NumbersCustomModexp
+
+        class TestIntegerCustomModexp(TestIntegerBase):
+            def setUp(self):
+                self.Numbers = NumbersCustomModexp
+                self.Integer = NumbersCustomModexp.Integer
+                TestIntegerBase.setUp(self)
+
+        tests += list_test_cases(TestIntegerCustomModexp)
+    except (ImportError, OSError) as e:
+        import sys
+        sys.stdout.write("Skipping custom modexp tests (%s)\n" % str(e) )
+
     tests += list_test_cases(TestIntegerGeneric)
     return tests
 
